package keystash.sign;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import keystash.util.Convert;
import keystash.util.Time;
import keystash.util.Constants;

public class Transaction {

    static final long genesisAccount = Constants.genesisAccount;
    static volatile Time time;
    static int timestamp;
    static final int getFlags = Constants.getFlags;
    static final int ecBlockHeight = Constants.ecBlockHeight;
    static final long ecBlockId = Constants.ecBlockId;
    static final byte version = Constants.version;

    public static byte[] getBytes(short deadline, byte[] senderPublicKey, long recipientId, long amountNQT, long feeNQT, byte[] signature) {
        try {
            time = new Time.EpochTime();
            timestamp = Convert.getEpochTime();
            ByteBuffer buffer = ByteBuffer.allocate(Constants.payloadLength);
            buffer.order(ByteOrder.LITTLE_ENDIAN);
            buffer.put((byte) Constants.type);
            buffer.put((byte) version);
            buffer.putInt(timestamp);
            buffer.putShort(deadline);
            buffer.put(senderPublicKey);
            buffer.putLong(true ? recipientId : genesisAccount);
            buffer.putLong(amountNQT);
            buffer.putLong(feeNQT);
            buffer.put(new byte[32]);
            buffer.put(signature != null ? signature : new byte[64]);
            if (version > 0) {

                buffer.putInt(getFlags);
                buffer.putInt(ecBlockHeight);
                buffer.putLong(ecBlockId);

            }

            return buffer.array();
        } catch (RuntimeException e) {
            System.err.println("Failed to get transaction bytes for transaction.");
            throw e;
        }

    }

    public static int getEpochTime() {
        return time.getTime();
    }

}
